/*
 *  AccelHandler.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's Computers
 * and Cognition Group. For more information, see 
 * <a href="http://www.cognition.olin.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@olin.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Copyright (C) 2002-2003 Franklin W. Olin College of Engineering.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

/**
 * This class contains a Thread that continuously prompts the
 * user's class for an acceleration and stores the result in a {@link Coord}.
 * Other objects (such as {@link DotPanel}) can monitor the <code>Coord</code>
 * for updates.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2002-2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @version $Id: AccelHandler.java,v 1.7 2004/02/09 20:55:03 gus Exp $
 * @see Accelerator
 * @see DotPanel
 * @see Coord
 */
public class AccelHandler implements Runnable {
  /** Position control mode */  
  public static final int POSMODE = 1;
  /** Velocity control mdde */
  public static final int VELMODE = 2;
  /** Acceleration control mode */
  public static final int ACCELMODE = 3;
    
  private Accelerator Accel;    
  private Coord myCoord;
  private Coord otherCoord;
    
  /**
   * Creates a new AccelHandler.  Does not start a new Thread.
   *
   * @param Accel Provides behavior based on student code
   * @param myCoord Our position for this Dimension
   * @param otherCoord Our position in the other Dimension
   */
  public AccelHandler(Accelerator Accel, Coord myCoord, Coord otherCoord) {
    this.Accel = Accel;
    this.myCoord = myCoord;
    this.otherCoord = otherCoord;
  }
    
  /**
   * Changes the current Accelerator.
   *
   * @param Accel the new Accelerator
   */
  public synchronized void setAccel (Accelerator Accel) {
    // This method is synchronized with part of the while(true)
    // loop to keep people from switching the Accelerator while
    // it is being queried.

    System.out.println ("New Accelerator set.");
    this.Accel = Accel;
  }

  /**
   * Prompt the user's class for an accleration and pass the info to
   * a Coord.
   */    
  public void run() {
      while (true) {
          synchronized (this) {
              double tmp = Accel.act( myCoord.getPos(), 
                                myCoord.getVel(), 
				otherCoord.getPos(), 
				otherCoord.getVel(), 
				myCoord.getMaxPos());
              double sign = tmp/Math.abs(tmp);
              tmp = (Math.abs(tmp) <= myCoord.getMaxPos()) ? 
                    tmp : myCoord.getMaxPos()*sign;
              myCoord.setValue(tmp);
          }
          try {
              Thread.sleep((int)(100*Spirograph.TIMESTEP));
              
              //Thread.yield();
          } catch (Exception e) {
              System.out.println ("Oops.");
          }
      }
  }
}

/*
 * $Log: AccelHandler.java,v $
 * Revision 1.7  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.6  2003/01/13 17:52:54  gus
 * Do some Bounds checking in the act method.
 *
 * Revision 1.5  2003/01/10 22:19:18  gus
 * Complete and update the javadocs
 *
 * Revision 1.4  2003/01/09 20:45:23  gus
 * Reindented the run method
 *
 * Revision 1.3  2003/01/09 20:36:25  gus
 * Added Javadoc and comments, updated Copyright
 *
 */