/*
 *  AdvEnv.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.awt.event.*;

/**
 * This class  creates the window that appears when the user clicks on
 * the Advanced Environment Options button. It has several Checkboxes
 * and a method to add listeners to those buttons.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: AdvEnv.java,v 1.3 2004/02/09 20:55:03 gus Exp $
 */
public class AdvEnv extends Frame {
  private Checkbox circularMode = new Checkbox("Circular");

  // This container holds the Textareas, the button and the labels
  // that are used to set the velocity.
  private Panel velCont = new Panel();
  private Button setVel = new Button("Set velocity");
  private DotPanel myPanel;
  private Checkbox bounceMode = new Checkbox("Bouncing");
  private Checkbox wrapMode = new Checkbox("Wraparound");

  /** Constructs the Advanced Environment Features menu with a default
   * font.
   *
   * @param thePanel The <code>DotPanel</code> to which the advanced options apply
   * @param xVel A <code>TextField</code> from which to get the velocity in
   *             the x direction
   * @param yVel A <code>TextField</code> from which to get the velocity in
   *             the y direction
   */
  public AdvEnv(TextField xVel, TextField yVel, DotPanel thePanel) {
	
    this(xVel, yVel, thePanel, Spirograph.DEFAULTFONT);
  }

  /** Specifies rules for switching the DotPanel to a circular mode,
   * switching the border behavior between wraparound and bounce,
   * setting position, velocity, and acceleration, and for closing the
   * window.
   *
   * @see DotPanel
   * @param xVel A <code>TextField</code> from which to get the velocity in
   *             the x direction
   * @param yVel A <code>TextField</code> from which to get the velocity in
   *             the y direction
   * @param thePanel The <code>DotPanel</code> to which the advanced options apply
   * @param f A custom font for the panel
   */
  public AdvEnv(TextField xVel, TextField yVel, DotPanel thePanel, Font f) {
	
    myPanel = thePanel;
	
    this.setFont(f);
    this.setTitle("Advanced Environment Features.");

    // border behavior modes
    circularMode.addItemListener(new ItemListener() {	    
      public void itemStateChanged (ItemEvent ie) {
	if (ie.getStateChange() == ItemEvent.SELECTED) {
	  if (myPanel.inEllipse()) {
	    myPanel.setCirc(true);
	    bounceMode.setState(true);
	    wrapMode.setState(false);
	  } else {
	    System.out.println ("You can't change to circular" +
				" mode if the ball would be" +
				" outside the circle.");
	    circularMode.setState(false);
	  }
	} else {
	  myPanel.setCirc(false);
	}
      }
    });

    bounceMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
	if(ie.getStateChange() == 1) {
	  if(!myPanel.getWrap()) {
	    myPanel.setBounce(true);
	  } else {
	    System.out.println("You can't turn bouncing on while wraparound is on.");
	    bounceMode.setState(false);
	  }
	} else {
	  myPanel.setBounce(false);
	}
      }
    });
    wrapMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
	if(ie.getStateChange() == 1) {
	  if(!myPanel.getBounce() && !myPanel.getCirc()) {
	    myPanel.setWrap(true);
	  } else {
	    System.out.println("Wraparound cannot be used with bouncing or circular mode.");
	    wrapMode.setState(false);
	  }
	} else {
	  myPanel.setWrap(false);
	}
      }
    });

    // position/vel/accel mode
    CheckboxGroup modeGroup = new CheckboxGroup();
    Checkbox posMode = new Checkbox("Position control.", modeGroup, true);
    posMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
	myPanel.setMode(AccelHandler.POSMODE);
      }
    });
    Checkbox velMode = new Checkbox("Velocity control.", modeGroup, false);
    velMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
	myPanel.setMode(AccelHandler.VELMODE);
      }
    });
    Checkbox accelMode = new Checkbox("Acceleration control.", modeGroup, false);
    accelMode.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent ie) {
	myPanel.setMode(AccelHandler.ACCELMODE);
      }
    });

    // Set up the velocity setting container
    velCont.setLayout(new FlowLayout(FlowLayout.LEFT));	
    velCont.add (new Label ("X vel:"));
    velCont.add (xVel);
    velCont.add (new Label ("Y vel:"));
    velCont.add (yVel);
    velCont.add (setVel);

    Button close = new Button ("Close this window");
    // Window disappears when the close Button is pushed
    close.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
	AdvEnv.this.setVisible(false);
      }
    });
	
    // The settings are implemented in a new Panel
    this.setLayout(new BorderLayout(2, 2));
    Panel top1 = new Panel();
    top1.setLayout(new GridLayout(3, 2));
    top1.add(circularMode);
    top1.add(posMode);
    top1.add(bounceMode);
    top1.add(velMode);
    top1.add(wrapMode);
    top1.add(accelMode);
    this.add("North", top1);
    this.add("Center", velCont);
    this.add("South", close);
    this.pack();
  }

  /** Add an actionListener to the setVel button
   * @param velSetter A listener that wants to know when the setVel button is pressed.
   */  
  public void addListener(ActionListener velSetter) {
    setVel.addActionListener(velSetter);
  }
}

/*
 * $Log: AdvEnv.java,v $
 * Revision 1.3  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.2  2003/01/10 22:17:18  gus
 * Complete the javadocs and clean up arguments for constructors
 *
 */