/*
 *  CodeBox.java
 *	Part of the Spirograph problem set.
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1998 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.awt.event.*;

/** This class pops up the Code editing area. The Code editing area
 * has two big text areas where the students can enter code, as well
 * as two smaller text areas (that initially start off hidden) that
 * can be used to enter fields.
 *
 * <P>This code also creates an instance of AdvProg, creates all of the
 * appropriate listeners and passes them to the instance.
 * This code does not actually process and/or compile the user's code;
 * that is done by instances of CodeMerger, which are added to the
 * big buttons that say, "Load Horizontal Code" and "Load Vertical Code"
 *
 *  <p>Copyright &copy; 1998 Massachusetts Institute of Technology.<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 *
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @version $Id: CodeBox.java,v 1.5 2004/02/09 20:55:03 gus Exp $
 * @see AccelHandler
 * @see CodeMerger
 */
public class CodeBox extends Frame {

  /** the default width */
  static final int WIDTH = 800;
  /** the default height */
  static final int HEIGHT = 530;
  
  private TextArea hArea;
  private TextArea vArea;
  private TextArea hFields;
  private TextArea vFields;
  CodeMerger hMerger;
  CodeMerger vMerger;

  /** Construct a new CodeBox.  If no font is specified, use a default.
   * @param xHandler Animates motion in the X direction
   * @param yHandler Animates motion in the Y direction.
   */
  public CodeBox(AccelHandler xHandler, AccelHandler yHandler) {
	
    this(xHandler, yHandler, Spirograph.DEFAULTFONT);
  }
    
  /** Construct a CodeBox.  Use the specified font.
   * @param xHandler Animates motion in the x direction
   * @param yHandler Animates motion in the y direction
   * @param f A custom font for this component
   */
  public CodeBox(final AccelHandler xHandler, final AccelHandler yHandler, Font f) {

    hArea = new TextArea(15,50);
    vArea = new TextArea(15,50);
    hFields = new TextArea("",2,30,TextArea.SCROLLBARS_VERTICAL_ONLY);
    vFields = new TextArea("",2,30,TextArea.SCROLLBARS_VERTICAL_ONLY);

    hMerger = new CodeMerger("Horizontal", new TextAreaCodeSource(hArea, hFields), xHandler, yHandler);
    vMerger = new CodeMerger("Vertical", new TextAreaCodeSource(vArea, vFields), yHandler, xHandler);
                     
    this.setFont(f);
    this.setTitle("Spirograph Code Editor");
	
    Panel top = new Panel();
    top.setLayout(new BorderLayout(10,10));
    top.add ("West", new MultiLabel (
				     "Note: The variable pos holds the position of the dot along each " +
				     "axis, and the variable vel holds the velocity of the dot\n " +
				     "along each axis. The variables otherPos and otherVel hold "+
				     "the position and velocity of the dot along the other axis."));
    Button quit = new Button("Quit");
    quit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
	System.exit(0);
      }
    });
    top.add("Center", quit);
	
    // Set up fields and labels for horiz area
    Label hFieldLabel=new Label ("Enter horizontal rule fields here.");
	
    Panel hTop = new Panel();
    hTop.setLayout(new BorderLayout(10,10));
    hTop.add("North", hFieldLabel);
    hTop.add("South", hFields);

    Panel hMid = new Panel();
    hMid.setLayout(new BorderLayout(10,10));
    hMid.add("North", new Label ("Enter horizontal rule below:"));
    hMid.add("South", hArea );

    Checkbox hBoth = new Checkbox( "Use this rule for both dimensions", false );
    Button hDone = new Button("Compile and Load Horizontal Code");
    hDone.addActionListener(hMerger);
    Panel hBot = new Panel();
    hBot.setLayout(new BorderLayout(10,3));
    hBot.add("North", hBoth );
    hBot.add("South", hDone );
	
    // Add fields and labels, text area and Done button to horiz area
    Panel hContainer = new Panel();	
    hContainer.setLayout (new BorderLayout(10,10));
    hContainer.add("North", hTop);
    hContainer.add("Center", hMid);
    hContainer.add("South", hBot);

    // Set up fields and labels for vertical area
    Label vFieldLabel=new Label ("Enter vertical rule fields here.");
	
    Panel vTop = new Panel();
    vTop.setLayout(new BorderLayout(10,10));
    vTop.add("North", vFieldLabel);
    vTop.add("South", vFields);

    Panel vMid = new Panel();
    vMid.setLayout(new BorderLayout(10,10));
    vMid.add("North",new Label ("Enter vertical rule below:"));
    vMid.add("South",vArea );

    Checkbox vBoth = new Checkbox( "Use this rule for both dimensions", false );
    Button vDone = new Button("Compile and Load Vertical Code");    	
    vDone.addActionListener(vMerger);
    Panel vBot = new Panel();
    vBot.setLayout(new BorderLayout(10,3));
    vBot.add("North", vBoth );
    vBot.add("South", vDone );

    // Add fields and labels, text area and Done button to vert area
    Panel vContainer = new Panel();	
    vContainer.setLayout (new BorderLayout(10,10));
    vContainer.add("North", vTop);
    vContainer.add("Center", vMid);
    vContainer.add("South", vBot);

    // Set up ItemListeners for "Advanced Programming"
    OneRuleListener hOnly = new OneRuleListener
      (hArea, vArea, hFields, vFields, hDone, vDone, vBoth, hMerger);
	
    OneRuleListener vOnly = new OneRuleListener
      (vArea, hArea, vFields, hFields, vDone, hDone, hBoth, vMerger);
	
    hBoth.addItemListener(hOnly);
    vBoth.addItemListener(vOnly);


    // Do final Layout
    this.setLayout(new BorderLayout(10,10));

    this.add ("West", hContainer);
    this.add ("East", vContainer);
    this.add ("South", top);
	
    this.setSize(CodeBox.WIDTH,CodeBox.HEIGHT);
    this.show();
  }

  private String liftText(String src, String start, String end)
  {
    int startpos = src.indexOf(start) + start.length();
    int endpos = src.indexOf(end);
    if(startpos > 0 && endpos > 0)
      return src.substring(startpos, endpos);
    else
      return "";
  }

  /** Accessor for the Vertical <code>CodeMerger</code>
   * @return The vertical (y) direction <code>CodeMerger</code>
   */  
  public CodeMerger getVMerger() {
      return vMerger;
  }
  
  /** Accessor for the Vertical <code>CodeMerger</code>
   * @return The vertical (y) direction <code>CodeMerger</code>
   */  
  public CodeMerger getHMerger() {
      return hMerger;
  }
}

/*
 * $Log: CodeBox.java,v $
 * Revision 1.5  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.4  2003/01/10 22:19:18  gus
 * Complete and update the javadocs
 *
 */