/*
 * MultiLabel.java
 * part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.util.*;

/** This class creates a self-sizing Canvas that prints a multi-line
 * String.  It can be given a specific font when created, or use a
 * default.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: MultiLabel.java,v 1.4 2003/01/15 17:36:10 gus Exp $
 * @see CodeBox
 */
public class MultiLabel extends Canvas {
    
    private static final int MARGINWIDTH = 10;
    
    private static final int MARGINHEIGHT = 10;

    private String lines[];
    private int numLines;
    private int maxWidth = 0;
    private int lineHeight;
    private int lineAscent;
        
    /**
     * Create a label capable of handling multiple lines of text with a default font.
     *
     * @param str The textual content for the label
     */    
    public MultiLabel(String str) {
	this(str, Spirograph.DEFAULTFONT);	
    }

    /** Create a label capable of handling multiple lines of text with a specified font.
     *
     * @param str The textual content for the label
     * @param f The desired font
     */    
    public MultiLabel(String str, Font f) {	
	this.setFont(f);
	// Start off parsing str and measuring stuff.
	
	StringTokenizer t = new StringTokenizer(str, "\n");
	numLines = t.countTokens();	    
	lines = new String[numLines];
	for (int i = 0; i < numLines; i++) {
	    lines[i] = t.nextToken();
	}
	
	// Move on to font specific stuff.
	
	FontMetrics fm =this.getToolkit().getFontMetrics(this.getFont());
	lineHeight = fm.getHeight();
	lineAscent = fm.getAscent();
	for (int i = 0; i < numLines; i++) {
	    if (fm.stringWidth(lines[i]) > maxWidth) {
		maxWidth = fm.stringWidth(lines[i]);
	    }
	}
	
	this.setSize(maxWidth + 2*MultiLabel.MARGINWIDTH,
		     lineHeight * numLines + 2*MultiLabel.MARGINHEIGHT);
    }

    /** Overides the paint method of Canvas to do our drawing.
     * @param g The graphics context of this component.
     */    
    public void paint(Graphics g) {
	int y = MultiLabel.MARGINHEIGHT + lineAscent;
	
	for (int i = 0; i < numLines; i++) {
	    g.drawString(lines[i], MultiLabel.MARGINWIDTH, y);
	    y += lineHeight;
	}
    }
}    

/*
 * $Log: MultiLabel.java,v $
 * Revision 1.4  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */