/*
 * OneRuleListener.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.event.*;
import java.awt.*;

/** This implements a listener that is created by {@link CodeBox } and passed
 * to {@link AdvEnv}. It enables or disables the approprite text areas and
 * fields so that when only one rule is being used the other rule is
 * grayed out.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: OneRuleListener.java,v 1.4 2004/02/09 20:55:03 gus Exp $
 * @see CodeMerger
 * @see CodeBox
 */
public class OneRuleListener implements ItemListener {
    TextArea onTA, offTA, onFields, offFields;
    Button onButton, offButton;
    Checkbox offCheckbox;
    CodeMerger myMerger;
    
    /**
     * Create an object that monitors the state of the specified components and
     * enables or disables them as needed.
     *
     * @param onTA The code text area that remains on
     * @param offTA The code text area that is turned off
     * @param onFields The fields text area that remains on
     * @param offFields The fields text area that is turned off
     * @param onButton The button that remains on
     * @param offButton The button that is turned off
     * @param offCheckbox The other check box wich gets turned off
     * @param myMerger The code merger that is to be used for both rules
     */    
    public OneRuleListener(TextArea onTA, TextArea offTA, TextArea onFields, TextArea offFields, Button onButton, Button offButton, Checkbox offCheckbox, CodeMerger myMerger) {
	this.onTA = onTA;
	this.offTA = offTA;
	this.onFields = onFields;
	this.offFields = offFields;
	this.onButton = onButton;
	this.offButton = offButton;
        this.offCheckbox = offCheckbox;
	this.myMerger = myMerger;
    }

    /**
     * Examine the supplied event and disable components if the item has been selected.
     *
     * @param ie An item event from the component we are listening to.
     */    
    public void itemStateChanged(ItemEvent ie) {
        if ( ie.getStateChange() == ItemEvent.SELECTED )
        {
	    onTA.setEnabled(true);
	    offTA.setEnabled(false);
	    onFields.setEnabled(true);
	    offFields.setEnabled(false);
	    onButton.setEnabled(true);
	    offButton.setEnabled(false);
            offCheckbox.setState(false);
            offCheckbox.setEnabled(false);
	    myMerger.setBoth(true);
        } else {
	    onTA.setEnabled(true);
	    offTA.setEnabled(true);
	    onFields.setEnabled(true);
	    offFields.setEnabled(true);
	    onButton.setEnabled(true);
	    offButton.setEnabled(true);
            offCheckbox.setEnabled(true);
	    myMerger.setBoth(false);
        }
    }
}

/*
 * $Log: OneRuleListener.java,v $
 * Revision 1.4  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.3  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */