/*
 * RecipieView.java
 *
 * Created on December 9, 2002, 4:47 PM
 */

package spirograph;

import spirograph.Recipie;

import java.awt.Color;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import java.awt.event.KeyEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.ActionListener;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;

import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;


/**
 * This component displays the text of a {@link Recipie}, and it's usage status.
 * It registers as a {@link RecipieListener} and a {@link EtchControlListener}
 * to actively monitor and react to changes in the <code>Recipie</code> fields or
 * in the assignment of the displayed <code>Recipie</code> to the vertical or
 * horizontal axes.
 *
 * <p>Copyright &copy; 2003 Franklin W. Olin College of Engineering.</p>
 *
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: RecipieView.java,v 1.6 2004/02/09 20:55:03 gus Exp $
 */
public class RecipieView extends javax.swing.JPanel implements RecipieListener, EtchControlListener{
    
    private JTextArea statementTextArea;
    private JPanel fieldPanel;
    private JPanel hvCheckPanel;
    private JPanel statementPanel;
    private JTextArea fieldTextArea;
    private JScrollPane fieldScrollPane;
    private JLabel statementLabel;
    private JLabel fieldLabel;
    private JPanel recipieButtonPanel;
    private JPanel recipieControlPanel;
    private JScrollPane statementScrollPane;
    private JToggleButton fieldsToggleButton;
    private JLabel statusHeaderLabel;
    private JLabel statusLabel;
    
    private Recipie theRecipie = Recipie.nullRecipie;
    // End of variables declaration

    /** Creates a new instance of <code>RecipieView</code>. The new instance will
     * view {@link Recipie#nullRecipie} until a new <code>Recipie</code> is set.
     * This makes it safe to add this component and initialize it at a later time.
     * Without this feature, there is a danger of of a a call to
     * {@link RecipieView#recipieUpdated} throwing a
     * <code>NullPointerException</code> before the <code>Recipie</code> is set.
     */
    public RecipieView() {
        initComponents();
        setBorder(new javax.swing.border.EtchedBorder());
    }
    
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        fieldPanel = new JPanel();
        fieldPanel.setVisible(false);
        fieldLabel = new JLabel();
        fieldScrollPane = new JScrollPane();
        fieldTextArea = new JTextArea();
        statementPanel = new JPanel();
        statementLabel = new JLabel();
        statementScrollPane = new JScrollPane();
        statementTextArea = new JTextArea();
        recipieControlPanel = new JPanel();
        recipieButtonPanel = new JPanel();
        fieldsToggleButton = new JToggleButton();
        hvCheckPanel = new JPanel();
        statusHeaderLabel = new JLabel();
        statusLabel = new JLabel();

        setLayout(new GridBagLayout());

        fieldPanel.setLayout(new BorderLayout());

        fieldLabel.setText("Enter your Recipie Fields Here");
        fieldLabel.setBorder(new EmptyBorder(new Insets(1, 1, 5, 1)));
        fieldPanel.add(fieldLabel, BorderLayout.NORTH);

        fieldTextArea.setColumns(30);
        fieldTextArea.setRows(4);
        fieldTextArea.setBorder(new BevelBorder(BevelBorder.LOWERED));
        fieldTextArea.addKeyListener( new KeyAdapter() {
            public void keyReleased(KeyEvent ke) {
                fieldTextAreaKeyReleased(ke);
            }
        });

        fieldScrollPane.setViewportView(fieldTextArea);

        fieldPanel.add(fieldScrollPane, BorderLayout.CENTER);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        add(fieldPanel, gridBagConstraints);

        statementPanel.setLayout(new BorderLayout());

        statementLabel.setText("Enter a Recipie Below");
        statementLabel.setBorder(new EmptyBorder(new Insets(10, 1, 5, 1)));
        statementPanel.add(statementLabel, BorderLayout.NORTH);

        statementTextArea.setColumns(30);
        statementTextArea.setRows(8);
        statementTextArea.setBorder(new BevelBorder(BevelBorder.LOWERED));
        statementTextArea.addKeyListener( new KeyAdapter() {
            public void keyReleased(KeyEvent ke) {
                statementTextAreaKeyReleased(ke);
            }
        });
        statementScrollPane.setViewportView(statementTextArea);

        statementPanel.add(statementScrollPane, BorderLayout.CENTER);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(statementPanel, gridBagConstraints);

        recipieControlPanel.setLayout(new GridBagLayout());

        recipieButtonPanel.setLayout(new GridBagLayout());

        fieldsToggleButton.setText("Add Fields");
        fieldsToggleButton.setToolTipText("Click to hide or unhide the" + 
            " fields box");
        fieldsToggleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                fieldsToggleButtonActionPerfomred(evt);
            }
        });
        
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        recipieButtonPanel.add(fieldsToggleButton, gridBagConstraints);


        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        recipieControlPanel.add(recipieButtonPanel, gridBagConstraints);

        hvCheckPanel.setLayout(new GridBagLayout());

        statusHeaderLabel.setText("This recipe used for");

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 2.0;
        hvCheckPanel.add(statusHeaderLabel, gridBagConstraints);

        statusLabel.setText("None");
        statusLabel.setFont(new java.awt.Font("Dialog", 1, 16));
        statusLabel.setForeground(Color.red);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.weightx = 1.0;
        hvCheckPanel.add(statusLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        recipieControlPanel.add(hvCheckPanel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(10, 0, 0, 0);
        gridBagConstraints.weightx = 1.0;
        add(recipieControlPanel, gridBagConstraints);


    }

    private void fieldTextAreaKeyReleased(KeyEvent ke) {
        theRecipie.setFields(fieldTextArea.getText());
    }

    private void statementTextAreaKeyReleased(KeyEvent ke) {
        theRecipie.setCode(statementTextArea.getText());
    }
    
    private void fieldsToggleButtonActionPerfomred(ActionEvent evt) {
        fieldPanel.setVisible(!fieldPanel.isVisible());
    }
    
    // no javadoc - inherits from EtchControlListener
    public void etchControlUpdated(javax.swing.event.ChangeEvent ce) {
        boolean isVert = ((EtchControl)ce.getSource()).isVertRecipie(theRecipie);
        boolean isHorz = ((EtchControl)ce.getSource()).isHorzRecipie(theRecipie);
        if (isVert && isHorz) {
            statusLabel.setText("Both");
            statusLabel.setForeground(Color.blue);
            if (getParent() instanceof JTabbedPane) {
                ((JTabbedPane) getParent()).setForegroundAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                Color.blue);
                
                ((JTabbedPane) getParent()).setToolTipTextAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                "The recipie currently used for both vertical and horizontal positioning of the pen");
                
            }
        }
        if (isVert && !isHorz) {
            statusLabel.setText("Vertical");
            statusLabel.setForeground(new Color(0,128,0));
            if (getParent() instanceof JTabbedPane) {
                ((JTabbedPane) getParent()).setForegroundAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                new Color(0,128,0));
                ((JTabbedPane) getParent()).setToolTipTextAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                "The recipie currently used for vertical positioning of the pen");

            }
        }
        if (!isVert && isHorz) {
            statusLabel.setText("Horizontal");
            statusLabel.setForeground(Color.magenta);
            if (getParent() instanceof JTabbedPane) {
                ((JTabbedPane) getParent()).setForegroundAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                Color.magenta);
                ((JTabbedPane) getParent()).setToolTipTextAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                "The recipie currently used for horizontal positioning of the pen");
            }
        }
        if (!isVert && !isHorz) {
            statusLabel.setText("None");
            statusLabel.setForeground(Color.red);
            if (getParent() instanceof JTabbedPane) {
                ((JTabbedPane) getParent()).setForegroundAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                Color.black);
                ((JTabbedPane) getParent()).setToolTipTextAt(
                ((JTabbedPane) getParent()).indexOfComponent(this), 
                "A recipie not currently in use");
            }
        }
            
    }

    // no javadoc - inherits from RecipieListener
    public void recipieUpdated(javax.swing.event.ChangeEvent ce) {
        int caretPos = statementTextArea.getCaretPosition();
        statementTextArea.setText(((Recipie) ce.getSource()).getCode());
        statementTextArea.setCaretPosition(caretPos);
        caretPos = fieldTextArea.getCaretPosition();
        fieldTextArea.setText(((Recipie) ce.getSource()).getFields());
        fieldTextArea.setCaretPosition(caretPos);
    }
    
    /**
     * Set the {@link Recipie} for this view to display.
     *
     * @param r The <code>Recipie</code> to View
     */    
    public void setRecipie(Recipie r) {
        theRecipie = r;
        r.addRecipieListener(this);
    }
    
    /**
     * Get a reference to the {@link Recipie} object currently displayed by this view.
     *
     * @return The <code>Recipie</code> being viewed.
     */    
    public Recipie getRecipie() {
        return theRecipie;
    }

    /** Identify the {@link Recipie} being displayed.
     * @return The unique identifier for the <code>Recipie</code> being displayed.
     */    
    public int getRecipieNum() {
        return theRecipie.getIdNum();
    }
    
  
    /*
     * $Log: RecipieView.java,v $
     * Revision 1.6  2004/02/09 20:55:03  gus
     * javadoc fixes
     *
     * Revision 1.5  2003/01/15 18:48:10  gus
     * finished Javadoc and removed unused method
     *
     * Revision 1.4  2003/01/15 18:25:59  gus
     * Some javadoc added, and editRecipie changed to more standard name of
     * setRecipie.
     *
     * Revision 1.3  2003/01/15 16:15:45  gus
     * accomodate changes in the Recipie class
     *
     * Revision 1.2  2002/12/20 21:49:08  gus
     * Connect the new GUI with the old backend.
     *
     * Revision 1.1  2002/12/19 18:55:51  gus
     * A new GUI for editing and displaying the code to control the behavior of the ball.
     *
     */
}
