/*
 * ResetMenu.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.awt.event.*;

/** This class creates a pop up menu that allows the user to reset different
 * things. All of the event handlers are passed from the DotFrame.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: ResetMenu.java,v 1.4 2003/01/17 18:40:22 gus Exp $
 * @see RecipieView
 */
public class ResetMenu extends Frame {
    Button lines = new Button("Click here to clear lines.");
    Button grav = new Button ("Click here to clear the gravity sources.");
    Button pos = new Button("Click here to stop the ball and move it " +
			     "back to the center.");
    Button all = new Button("Click here to do all of the above.");
    
    /** Create a new <code>ResetMenu</code> with the default font.
     */    
    public ResetMenu() {
	this(Spirograph.DEFAULTFONT);
    }

    // I use containers and a grid layout so that the bottom button will
    // span the entire width of the window. Doing it this way is easier
    // than using a GridBagLayout
    
    /** Create a new <code>ResetMenu</code> with the specified <code>Font</code>.
     * @param f The desired <code>Font</code>
     */    
    public ResetMenu(Font f) {

	this.setFont(f);
	this.setTitle("Reset Menu");
	this.setLayout(new GridLayout(4,1));
	this.add(lines);
	this.add(grav);
	this.add(pos);
	this.add(all);

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

	this.pack();
    }
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {
        ResetMenu.this.setVisible(false);
    }

    /** Add Listeners to each of the bottons in the reset menu. Generally these
     * buttons control objects in {@link DotFrame} and so the listeners are
     * generated there and added via this method.
     * @param lList The listener for the line deletion button
     * @param gList The listener for the gravity deletion button
     * @param pList The listener for the button to stop the ball and center it
     * @param aList The listener for the all of the above button
     */    
    public void addListeners(ActionListener lList, ActionListener gList,
			     ActionListener pList, ActionListener aList) {
	lines.addActionListener(lList);
	grav.addActionListener(gList);
	pos.addActionListener(pList);
	all.addActionListener(aList);
    }

}

/*
 * $Log: ResetMenu.java,v $
 * Revision 1.4  2003/01/17 18:40:22  gus
 * Adjust the grid layout to accomodate the fact that we nolonger have a
 * close this window button.
 *
 * Revision 1.3  2003/01/17 18:34:55  gus
 * Shrink the size of and simplify this window. Activate the close widow icon, and
 * add javadocs too
 *
 * Revision 1.2  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */