/*
 * SetPosGravListener.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.awt.event.*;

/** This code implements a listener which listens to the DotPanel and
 * waits for mouse clicks. Depending on what "mode" it is in, the
 * SetPosGravListener will either add in a new gravitational source,
 * reposition the dot or do nothing. The mode of this listener is change
 * by some of the button listeners in the AdvEnv instance.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: SetPosGravListener.java,v 1.3 2003/01/17 19:31:08 gus Exp $
 * @see Coord
 * @see DotPanel
 */
public class SetPosGravListener implements MouseListener {
    /**
     * Signifies that no action should be taken when the mouse is clicked in the
     * DotPanel.
     *
     */    
    public static final int NONE = 0;
    /** Signifies that the ball position should be set when the mouse is clicked in the
     * DotPanel.
     */    
    public static final int POS = 1;
    /**
     * Signifies that a gravity point should be added when the mouse is clicked in the
     * DotPanel.
     */    
    public static final int GRAV = 2;

    private Coord xCoord;
    private Coord yCoord;
    private DotPanel myPanel;
    private int mode = SetPosGravListener.POS;
    
    /** Creates a new SetPosGravListener that knows about a DotPanel and it's
     * associated Coordinates.
     * @param xCoord The x coordinate
     * @param yCoord the y coordinate.
     * @param myPanel The dot panel for which we will be a mouse listener.
     */    
    public SetPosGravListener(Coord xCoord, Coord yCoord, DotPanel myPanel){
	this.xCoord = xCoord;
	this.yCoord = yCoord;
	this.myPanel = myPanel;
    }

    // If the mouse is pressed do something depending on what mode you're in.
    /** Perform our mode dependant behavior. In GRAV mode, add a gravity point. In
     * POS mode adjust the position of the ball. In NONE mode do nothing.
     * @param e The mouse event we are to react to
     */    
    public void mousePressed(MouseEvent e) {
	double newX = e.getX()-myPanel.curWidth()/2;
	double newY = myPanel.curHeight()/2-e.getY();

	if (mode == SetPosGravListener.POS) {

	    if ((!(myPanel.getCirc())) ||
		((Math.pow(2*newX/myPanel.curWidth(),2) +
		  Math.pow(2*newY/myPanel.curHeight(),2)) < 1)) {
		// Only let the user place inside the circle if in
		// circular mode
		
		System.out.println ("Ball moved to " + newX + ":" + newY);
		
		xCoord.setPos(newX);
		yCoord.setPos(newY);
	    } else {
		System.out.println ("You can't place the ball there.");
	    }
	} else if (mode == SetPosGravListener.GRAV) {

	    System.out.println ("Adding grav spot at: " + newX + ":" + newY);

	    myPanel.addGrav((int)newX, (int)newY);
	}
	    
	myPanel.paintBuf();
    }

    // Some methods that the interface requires.
    /** Stub to fulfill MouseListener.
     * @param e A MouseEvent
     */    
    public void mouseReleased(MouseEvent e) {}
    /** Stub to fulfill MouseListener.
     * @param e A MouseEvent
     */    
    public void mouseClicked(MouseEvent e) {}
    /** Stub to fulfill MouseListener.
     * @param e A MouseEvent
     */    
    public void mouseEntered(MouseEvent e) {}
    /** Stub to fulfill MouseListener.
     * @param e A MouseEvent
     */    
    public void mouseExited(MouseEvent e) {}

    /** Change the mode of operation of this listener. Apropriate values are defined
     * by the NONE POS and GRAV fields of this class.
     * @param mode The new mode for this listener.
     */    
    public void setMode (int mode) {
	this.mode = mode;
    }
    
}

/*
 * $Log: SetPosGravListener.java,v $
 * Revision 1.3  2003/01/17 19:31:08  gus
 * Add Javadocs
 *
 * Revision 1.2  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */