/*
 * SetVListener.java 
 * Part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

import java.awt.*;
import java.awt.event.*;

/** 
 * This class listens to the set velocity button in the AdvEnv
 * Frame and sets the velocity of the dot with whatever text is in
 * the TextArea.
 *
 * <p>Copyright &copy; 1998 Massachusetts Institute of Technology<br />
 * Copyright &copy; 2003 Franklin W. Olin College of Engineering</p>
 *
 * @author Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author Henry Wong, henryw@mit.edu
 * @author Patrick G. Heck, gus.heck@olin.edu
 * @version $Id: SetVListener.java,v 1.4 2004/02/09 20:55:03 gus Exp $
 * @see Coord
 * @see DotPanel
 */
public class SetVListener implements ActionListener {
  private TextField xVel;
  private TextField yVel;
  private Coord xCoord;
  private Coord yCoord;
    
  /** Creates a new SetVListener to listen to the velocity setter button in AdvEnv.
   * @param xVel The x velocity
   * @param yVel The y velocity
   * @param xCoord The coordiante along the X axis
   * @param yCoord The coordiante along the Y axis
   */  
  public SetVListener(TextField xVel, TextField yVel, Coord xCoord, Coord yCoord) {
    this.xVel = xVel;
    this.yVel = yVel;
    this.xCoord = xCoord;
    this.yCoord = yCoord;
  }
    
  /** Respond to the button and set the Velocity.
   * @param ae The Event from the set velocity button
   */  
  public void actionPerformed(ActionEvent ae) {
    double newX;
    double newY;

    try {
      newX = Double.valueOf(xVel.getText()).doubleValue();
      newY = Double.valueOf(yVel.getText()).doubleValue();
    } catch (NumberFormatException e) {
      System.out.println ("Error setting velocity");
      return;
    }	
	
    // There is a maximum velocity that I set, b/c otherwise if the
    // ball is moving too fast things start to get really messy and
    // inaccurate.
	
    if (Math.sqrt(Math.pow(newX,2) + Math.pow(newY,2)) >
	Spirograph.MAXVEL) {
      newX = Spirograph.MAXVEL*newX/(newX+newY);
      newY = Spirograph.MAXVEL*newY/(newX+newY);
      xCoord.setVel(newX);
      yCoord.setVel(newY);
      xVel.setText(String.valueOf(Math.round(newX)));
      yVel.setText(String.valueOf(Math.round(newY)));
	    
      System.out.println ("Velocity too high, being coerced down.");
    }
	
    xCoord.setVel(newX);
    yCoord.setVel(newY);
    System.out.println ("Velocity set");

  }
}

/*
 * $Log: SetVListener.java,v $
 * Revision 1.4  2004/02/09 20:55:03  gus
 * javadoc fixes
 *
 * Revision 1.3  2003/01/17 20:49:07  gus
 * Added Javadoc
 *
 * Revision 1.2  2003/01/15 17:36:10  gus
 * adding log keywords to files that don't have them
 *
 */