/*
 * Utils.java
 * part of the Spirograph problem set
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1996 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */

package spirograph;

/**
 * This class has some all purpose utilities that will probably find their
 * way into cs101.util. I didn't really hunt through cs101.util before I
 * wrote these, so better versions of these may already exist somewhere. <p>
 *
 * Copyright (c) 1998 Massachusetts Institute of Technology
 *
 * @author  Luis Sarmenta, lfgs@cag.lcs.mit.edu
 * @author  Henry Wong, henryw@mit.edu
 * @version  $Id: Util.java,v 1.1.1.1 2002/06/05 21:56:35 root Exp $
 *
 */

import java.io.*;

public abstract class Util {

    // Exit status of successful compilation
    public static final int SUCCESS = 0;
    
    // System command to compile a program
    public static final String JAVAC = "javac";

    /**
     * This method reads the specified file off of the current directory
     * and returns it as a String.
     */
    
    public static String readFile(String filename) {
	File theFile = new File(filename);
	FileInputStream myFIS = null;
	String content = "";
	BufferedReader br;
	boolean moreFile = true;
	
	if (theFile.exists() && theFile.canRead()) {
	    System.out.println ("Opening " + filename + " file.");
	    try {
		myFIS = new FileInputStream(theFile);
	    } catch (FileNotFoundException e) {
		System.out.println ("Error, file not found.");
		return "";
	    }
	    
	    br = new BufferedReader(new InputStreamReader(myFIS));

	    // Keep reading the file until hit an EOF, but don't add the
	    // EOF to the String.
	    while (moreFile) {
		try {
		    String temp = br.readLine();
		    if (temp == null) {
			moreFile = false;
		    } else {
			content = content + temp + "\n";
		    }
		} catch (IOException e) {
		    System.out.println ("Error reading.");
		}
	    }
	    return (content);
	}
	return "";
    }	

    /**
     * This routine takes a String and writes it to a file in the current
     * directory. If the file already exists it is overwritten.
     */
    
    public static void writeFile (String filename, String message) {
	File myFile = new File(filename);
	FileOutputStream myFOS = null;
	
	try {
	    myFOS = new FileOutputStream(myFile);
	} catch (IOException e) {
	    System.out.println ("Error creating FileOutputStream");
	    return;
	}		
		
	if (!myFile.exists()) {
	    System.out.println ("Creating new " + filename + " file.");
	} else if (myFile.isFile() &&
		   myFile.canWrite()) {
	    System.out.println ("Editing existing " + filename + " file.");
	} else {
	    System.out.println ("Error. Can't write to " + filename);
	    try {
		myFOS.close();
	    } catch (IOException e) {}
	    return;
	}
		   
	try {
	    myFOS.write(message.getBytes());
	} catch (IOException e) {
	    System.out.println ("Error writing to file.");
	}
	try {
	    myFOS.close();
	} catch (IOException ioe) {}
	return;
    }

    /**
     * This routine takes a string and compiles that file in the current
     * directory. If there are any error then the method will return a
     * non-zero exit status and the errors will be piped to the screen.
     */

    /**
     * This method calculates the distance between any two points in 2D space
     */
    public static double dist(double ax, double ay, double bx, double by) {
	return Math.sqrt(Math.pow(ax-bx,2)+Math.pow(ay-by,2));
    }

    /**
     * This method calculate whether or not a given position is inside a
     * given ellipse.
     */
    
    public static boolean inEllipse(double x, double y, double a, double b) {
	return (Math.pow(x/a,2)+Math.pow(y/b,2) < 1);
    }

}
